@@ -1,12 +1,11 @@
 from enum import IntEnum
+
 import numpy as np
 
 from . import Geometry  # NOQA
-from . import lib
-from . import _geometry
+from . import _geometry, lib
 from .decorators import multithreading_enabled, requires_geos
 
-
 __all__ = [
     "GeometryType",
     "get_type_id",
@@ -26,6 +25,7 @@ __all__ = [
     "get_interior_ring",
     "get_geometry",
     "get_parts",
+    "get_rings",
     "get_precision",
     "set_precision",
 ]
@@ -49,7 +49,7 @@ class GeometryType(IntEnum):
 
 
 @multithreading_enabled
-def get_type_id(geometry):
+def get_type_id(geometry, **kwargs):
     """Returns the type ID of a geometry.
 
     - None (missing) is -1
@@ -65,6 +65,9 @@ def get_type_id(geometry):
     Parameters
     ----------
     geometry : Geometry or array_like
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -77,11 +80,11 @@ def get_type_id(geometry):
     >>> get_type_id([Geometry("POINT (1 2)"), Geometry("POINT (1 2)")]).tolist()
     [0, 0]
     """
-    return lib.get_type_id(geometry)
+    return lib.get_type_id(geometry, **kwargs)
 
 
 @multithreading_enabled
-def get_dimensions(geometry):
+def get_dimensions(geometry, **kwargs):
     """Returns the inherent dimensionality of a geometry.
 
     The inherent dimension is 0 for points, 1 for linestrings and linearrings,
@@ -91,6 +94,9 @@ def get_dimensions(geometry):
     Parameters
     ----------
     geometry : Geometry or array_like
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     Examples
     --------
@@ -105,11 +111,11 @@ def get_dimensions(geometry):
     >>> get_dimensions(None)
     -1
     """
-    return lib.get_dimensions(geometry)
+    return lib.get_dimensions(geometry, **kwargs)
 
 
 @multithreading_enabled
-def get_coordinate_dimension(geometry):
+def get_coordinate_dimension(geometry, **kwargs):
     """Returns the dimensionality of the coordinates in a geometry (2 or 3).
 
     Returns -1 for not-a-geometry values.
@@ -117,6 +123,9 @@ def get_coordinate_dimension(geometry):
     Parameters
     ----------
     geometry : Geometry or array_like
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     Examples
     --------
@@ -127,11 +136,11 @@ def get_coordinate_dimension(geometry):
     >>> get_coordinate_dimension(None)
     -1
     """
-    return lib.get_coordinate_dimension(geometry)
+    return lib.get_coordinate_dimension(geometry, **kwargs)
 
 
 @multithreading_enabled
-def get_num_coordinates(geometry):
+def get_num_coordinates(geometry, **kwargs):
     """Returns the total number of coordinates in a geometry.
 
     Returns 0 for not-a-geometry values.
@@ -139,6 +148,9 @@ def get_num_coordinates(geometry):
     Parameters
     ----------
     geometry : Geometry or array_like
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     Examples
     --------
@@ -151,11 +163,11 @@ def get_num_coordinates(geometry):
     >>> get_num_coordinates(None)
     0
     """
-    return lib.get_num_coordinates(geometry)
+    return lib.get_num_coordinates(geometry, **kwargs)
 
 
 @multithreading_enabled
-def get_srid(geometry):
+def get_srid(geometry, **kwargs):
     """Returns the SRID of a geometry.
 
     Returns -1 for not-a-geometry values.
@@ -163,6 +175,9 @@ def get_srid(geometry):
     Parameters
     ----------
     geometry : Geometry or array_like
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -177,17 +192,20 @@ def get_srid(geometry):
     >>> get_srid(with_srid)
     4326
     """
-    return lib.get_srid(geometry)
+    return lib.get_srid(geometry, **kwargs)
 
 
 @multithreading_enabled
-def set_srid(geometry, srid):
+def set_srid(geometry, srid, **kwargs):
     """Returns a geometry with its SRID set.
 
     Parameters
     ----------
     geometry : Geometry or array_like
     srid : int
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -202,20 +220,23 @@ def set_srid(geometry, srid):
     >>> get_srid(with_srid)
     4326
     """
-    return lib.set_srid(geometry, np.intc(srid))
+    return lib.set_srid(geometry, np.intc(srid), **kwargs)
 
 
 # points
 
 
 @multithreading_enabled
-def get_x(point):
+def get_x(point, **kwargs):
     """Returns the x-coordinate of a point
 
     Parameters
     ----------
     point : Geometry or array_like
         Non-point geometries will result in NaN being returned.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -228,17 +249,20 @@ def get_x(point):
     >>> get_x(Geometry("MULTIPOINT (1 1, 1 2)"))
     nan
     """
-    return lib.get_x(point)
+    return lib.get_x(point, **kwargs)
 
 
 @multithreading_enabled
-def get_y(point):
+def get_y(point, **kwargs):
     """Returns the y-coordinate of a point
 
     Parameters
     ----------
     point : Geometry or array_like
         Non-point geometries will result in NaN being returned.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -251,21 +275,22 @@ def get_y(point):
     >>> get_y(Geometry("MULTIPOINT (1 1, 1 2)"))
     nan
     """
-    return lib.get_y(point)
+    return lib.get_y(point, **kwargs)
 
 
 @requires_geos("3.7.0")
 @multithreading_enabled
-def get_z(point):
+def get_z(point, **kwargs):
     """Returns the z-coordinate of a point.
 
-    Requires at least GEOS 3.7.0.
-
     Parameters
     ----------
     point : Geometry or array_like
         Non-point geometries or geometries without 3rd dimension will result
         in NaN being returned.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -280,14 +305,14 @@ def get_z(point):
     >>> get_z(Geometry("MULTIPOINT Z (1 1 1, 2 2 2)"))
     nan
     """
-    return lib.get_z(point)
+    return lib.get_z(point, **kwargs)
 
 
 # linestrings
 
 
 @multithreading_enabled
-def get_point(geometry, index):
+def get_point(geometry, index, **kwargs):
     """Returns the nth point of a linestring or linearring.
 
     Parameters
@@ -295,6 +320,9 @@ def get_point(geometry, index):
     geometry : Geometry or array_like
     index : int or array_like
         Negative values count from the end of the linestring backwards.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -316,11 +344,11 @@ def get_point(geometry, index):
     >>> get_point(Geometry("POINT (1 1)"), 0) is None
     True
     """
-    return lib.get_point(geometry, np.intc(index))
+    return lib.get_point(geometry, np.intc(index), **kwargs)
 
 
 @multithreading_enabled
-def get_num_points(geometry):
+def get_num_points(geometry, **kwargs):
     """Returns number of points in a linestring or linearring.
 
     Returns 0 for not-a-geometry values.
@@ -330,6 +358,9 @@ def get_num_points(geometry):
     geometry : Geometry or array_like
         The number of points in geometries other than linestring or linearring
         equals zero.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -346,19 +377,22 @@ def get_num_points(geometry):
     >>> get_num_points(None)
     0
     """
-    return lib.get_num_points(geometry)
+    return lib.get_num_points(geometry, **kwargs)
 
 
 # polygons
 
 
 @multithreading_enabled
-def get_exterior_ring(geometry):
+def get_exterior_ring(geometry, **kwargs):
     """Returns the exterior ring of a polygon.
 
     Parameters
     ----------
     geometry : Geometry or array_like
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -371,11 +405,11 @@ def get_exterior_ring(geometry):
     >>> get_exterior_ring(Geometry("POINT (1 1)")) is None
     True
     """
-    return lib.get_exterior_ring(geometry)
+    return lib.get_exterior_ring(geometry, **kwargs)
 
 
 @multithreading_enabled
-def get_interior_ring(geometry, index):
+def get_interior_ring(geometry, index, **kwargs):
     """Returns the nth interior ring of a polygon.
 
     Parameters
@@ -383,6 +417,9 @@ def get_interior_ring(geometry, index):
     geometry : Geometry or array_like
     index : int or array_like
         Negative values count from the end of the interior rings backwards.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -397,11 +434,11 @@ def get_interior_ring(geometry, index):
     >>> get_interior_ring(Geometry("POINT (1 1)"), 0) is None
     True
     """
-    return lib.get_interior_ring(geometry, np.intc(index))
+    return lib.get_interior_ring(geometry, np.intc(index), **kwargs)
 
 
 @multithreading_enabled
-def get_num_interior_rings(geometry):
+def get_num_interior_rings(geometry, **kwargs):
     """Returns number of internal rings in a polygon
 
     Returns 0 for not-a-geometry values.
@@ -410,6 +447,9 @@ def get_num_interior_rings(geometry):
     ----------
     geometry : Geometry or array_like
         The number of interior rings in non-polygons equals zero.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -429,14 +469,14 @@ def get_num_interior_rings(geometry):
     >>> get_num_interior_rings(None)
     0
     """
-    return lib.get_num_interior_rings(geometry)
+    return lib.get_num_interior_rings(geometry, **kwargs)
 
 
 # collections
 
 
 @multithreading_enabled
-def get_geometry(geometry, index):
+def get_geometry(geometry, index, **kwargs):
     """Returns the nth geometry from a collection of geometries.
 
     Parameters
@@ -444,6 +484,9 @@ def get_geometry(geometry, index):
     geometry : Geometry or array_like
     index : int or array_like
         Negative values count from the end of the collection backwards.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     Notes
     -----
@@ -452,7 +495,7 @@ def get_geometry(geometry, index):
 
     See also
     --------
-    get_num_geometries
+    get_num_geometries, get_parts
 
     Examples
     --------
@@ -468,7 +511,7 @@ def get_geometry(geometry, index):
     >>> get_geometry(Geometry("POINT (1 1)"), 1) is None
     True
     """
-    return lib.get_geometry(geometry, np.intc(index))
+    return lib.get_geometry(geometry, np.intc(index), **kwargs)
 
 
 def get_parts(geometry, return_index=False):
@@ -482,14 +525,18 @@ def get_parts(geometry, return_index=False):
     Parameters
     ----------
     geometry : Geometry or array_like
-    return_index : bool, optional (default: False)
-        If True, will return a tuple of ndarrys of (parts, indexes), where indexes
+    return_index : bool, default False
+        If True, will return a tuple of ndarrays of (parts, indexes), where indexes
         are the indexes of the original geometries in the source array.
 
     Returns
     -------
     ndarray of parts or tuple of (parts, indexes)
 
+    See also
+    --------
+    get_geometry, get_rings
+
     Examples
     --------
     >>> get_parts(Geometry("MULTIPOINT (0 1, 2 3)")).tolist()
@@ -512,8 +559,63 @@ def get_parts(geometry, return_index=False):
     return _geometry.get_parts(geometry)[0]
 
 
+def get_rings(geometry, return_index=False):
+    """Gets rings of Polygon geometry object.
+
+    For each Polygon, the first returned ring is always the exterior ring
+    and potential subsequent rings are interior rings.
+
+    If the geometry is not a Polygon, nothing is returned (empty array for
+    scalar geometry input or no element in output array for array input).
+
+    Parameters
+    ----------
+    geometry : Geometry or array_like
+    return_index : bool, default False
+        If True, will return a tuple of ndarrays of (rings, indexes), where
+        indexes are the indexes of the original geometries in the source array.
+
+    Returns
+    -------
+    ndarray of rings or tuple of (rings, indexes)
+
+    See also
+    --------
+    get_exterior_ring, get_interior_ring, get_parts
+
+    Examples
+    --------
+    >>> polygon_with_hole = Geometry("POLYGON ((0 0, 0 10, 10 10, 10 0, 0 0), \
+(2 2, 2 4, 4 4, 4 2, 2 2))")
+    >>> get_rings(polygon_with_hole).tolist()
+    [<pygeos.Geometry LINEARRING (0 0, 0 10, 10 10, 10 0, 0 0)>,
+     <pygeos.Geometry LINEARRING (2 2, 2 4, 4 4, 4 2, 2 2)>]
+
+    With ``return_index=True``:
+
+    >>> polygon = Geometry("POLYGON ((0 0, 2 0, 2 2, 0 2, 0 0))")
+    >>> rings, index = get_rings([polygon, polygon_with_hole], return_index=True)
+    >>> rings.tolist()
+    [<pygeos.Geometry LINEARRING (0 0, 2 0, 2 2, 0 2, 0 0)>,
+     <pygeos.Geometry LINEARRING (0 0, 0 10, 10 10, 10 0, 0 0)>,
+     <pygeos.Geometry LINEARRING (2 2, 2 4, 4 4, 4 2, 2 2)>]
+    >>> index.tolist()
+    [0, 1, 1]
+    """
+    geometry = np.asarray(geometry, dtype=np.object_)
+    geometry = np.atleast_1d(geometry)
+
+    if geometry.ndim != 1:
+        raise ValueError("Array should be one dimensional")
+
+    if return_index:
+        return _geometry.get_parts(geometry, extract_rings=True)
+
+    return _geometry.get_parts(geometry, extract_rings=True)[0]
+
+
 @multithreading_enabled
-def get_num_geometries(geometry):
+def get_num_geometries(geometry, **kwargs):
     """Returns number of geometries in a collection.
 
     Returns 0 for not-a-geometry values.
@@ -523,6 +625,9 @@ def get_num_geometries(geometry):
     geometry : Geometry or array_like
         The number of geometries in points, linestrings, linearrings and
         polygons equals one.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -538,12 +643,12 @@ def get_num_geometries(geometry):
     >>> get_num_geometries(None)
     0
     """
-    return lib.get_num_geometries(geometry)
+    return lib.get_num_geometries(geometry, **kwargs)
 
 
 @requires_geos("3.6.0")
 @multithreading_enabled
-def get_precision(geometry):
+def get_precision(geometry, **kwargs):
     """Get the precision of a geometry.
 
     If a precision has not been previously set, it will be 0 (double
@@ -555,6 +660,9 @@ def get_precision(geometry):
     Parameters
     ----------
     geometry : Geometry or array_like
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -570,12 +678,12 @@ def get_precision(geometry):
     >>> np.isnan(get_precision(None))
     True
     """
-    return lib.get_precision(geometry)
+    return lib.get_precision(geometry, **kwargs)
 
 
 @requires_geos("3.6.0")
 @multithreading_enabled
-def set_precision(geometry, grid_size, preserve_topology=False):
+def set_precision(geometry, grid_size, preserve_topology=False, **kwargs):
     """Returns geometry with the precision set to a precision grid size.
 
     By default, geometries use double precision coordinates (grid_size = 0).
@@ -603,9 +711,12 @@ def set_precision(geometry, grid_size, preserve_topology=False):
         geometry if precision grid size was not previously set). If this
         value is more precise than input geometry, the input geometry will
         not be modified.
-    preserve_topology : bool, optional (default: False)
+    preserve_topology : bool, default False
         If True, will attempt to preserve the topology of a geometry after
         rounding coordinates.
+    **kwargs
+        For other keyword-only arguments, see the
+        `NumPy ufunc docs <https://numpy.org/doc/stable/reference/ufuncs.html#ufuncs-kwargs>`_.
 
     See also
     --------
@@ -623,4 +734,4 @@ def set_precision(geometry, grid_size, preserve_topology=False):
     True
     """
 
-    return lib.set_precision(geometry, grid_size, preserve_topology)
+    return lib.set_precision(geometry, grid_size, preserve_topology, **kwargs)
