@@ -118,11 +118,11 @@ on error, 1 on success*/
 static char get_coordinates(GEOSContextHandle_t context, GEOSGeometry* geom,
                             PyArrayObject* out, npy_intp* cursor, int include_z) {
   int type = GEOSGeomTypeId_r(context, geom);
-  if ((type == 0) | (type == 1) | (type == 2)) {
+  if ((type == 0) || (type == 1) || (type == 2)) {
     return get_coordinates_simple(context, geom, out, cursor, include_z);
   } else if (type == 3) {
     return get_coordinates_polygon(context, geom, out, cursor, include_z);
-  } else if ((type >= 4) & (type <= 7)) {
+  } else if ((type >= 4) && (type <= 7)) {
     return get_coordinates_collection(context, geom, out, cursor, include_z);
   } else {
     return 0;
@@ -140,7 +140,7 @@ static void* set_coordinates_simple(GEOSContextHandle_t context, GEOSGeometry* g
   GEOSGeometry* ret;
 
   /* Special case for POINT EMPTY (Point coordinate list cannot be 0-length) */
-  if ((type == 0) & (GEOSisEmpty_r(context, geom) == 1)) {
+  if ((type == 0) && (GEOSisEmpty_r(context, geom) == 1)) {
     return GEOSGeom_createEmptyPoint_r(context);
   }
 
@@ -288,11 +288,11 @@ correspondingly. Returns NULL on error,*/
 static void* set_coordinates(GEOSContextHandle_t context, GEOSGeometry* geom,
                              PyArrayObject* coords, npy_intp* cursor, int include_z) {
   int type = GEOSGeomTypeId_r(context, geom);
-  if ((type == 0) | (type == 1) | (type == 2)) {
+  if ((type == 0) || (type == 1) || (type == 2)) {
     return set_coordinates_simple(context, geom, type, coords, cursor, include_z);
   } else if (type == 3) {
     return set_coordinates_polygon(context, geom, coords, cursor, include_z);
-  } else if ((type >= 4) & (type <= 7)) {
+  } else if ((type >= 4) && (type <= 7)) {
     return set_coordinates_collection(context, geom, type, coords, cursor, include_z);
   } else {
     return NULL;
@@ -359,14 +359,15 @@ finish:
   return result;
 }
 
-PyObject* GetCoords(PyArrayObject* arr, int include_z) {
+PyObject* GetCoords(PyArrayObject* arr, int include_z, int return_index) {
   npy_intp coord_dim;
   NpyIter* iter;
   NpyIter_IterNextFunc* iternext;
   char** dataptr;
-  npy_intp cursor;
+  npy_intp cursor, i, geom_i;
   GeometryObject* obj;
   GEOSGeometry* geom;
+  PyArrayObject* index = NULL;
 
   /* create a coordinate array with the appropriate dimensions */
   npy_intp size = CountCoords(arr);
@@ -383,24 +384,42 @@ PyObject* GetCoords(PyArrayObject* arr, int include_z) {
   if (result == NULL) {
     return NULL;
   }
+  if (return_index) {
+    npy_intp dims_ind[1] = {size};
+    index = (PyArrayObject*)PyArray_SimpleNew(1, dims_ind, NPY_INTP);
+    if (index == NULL) {
+      Py_DECREF(result);
+      return NULL;
+    }
+  }
 
   /* Handle zero-sized arrays specially */
   if (PyArray_SIZE(arr) == 0) {
-    return (PyObject*)result;
+    if (return_index) {
+      PyObject* result_tpl = PyTuple_New(2);
+      PyTuple_SET_ITEM(result_tpl, 0, (PyObject*)result);
+      PyTuple_SET_ITEM(result_tpl, 1, (PyObject*)index);
+      return result_tpl;
+    } else {
+      return (PyObject*)result;
+    }
   }
 
   /* We use the Numpy iterator C-API here.
   The iterator exposes an "iternext" function which updates a "dataptr"
   see also: https://docs.scipy.org/doc/numpy/reference/c-api.iterator.html */
-  iter = NpyIter_New(arr, NPY_ITER_READONLY | NPY_ITER_REFS_OK, NPY_KEEPORDER,
+  iter = NpyIter_New(arr, NPY_ITER_READONLY | NPY_ITER_REFS_OK, NPY_CORDER,
                      NPY_NO_CASTING, NULL);
   if (iter == NULL) {
     Py_DECREF(result);
+    Py_XDECREF(index);
     return NULL;
   }
   iternext = NpyIter_GetIterNext(iter, NULL);
   if (iternext == NULL) {
     NpyIter_Deallocate(iter);
+    Py_DECREF(result);
+    Py_XDECREF(index);
     return NULL;
   }
   dataptr = NpyIter_GetDataPtrArray(iter);
@@ -410,9 +429,11 @@ PyObject* GetCoords(PyArrayObject* arr, int include_z) {
   /* We work with a "cursor" that tells the get_coordinates function where
   to write the coordinate data into the output array "result" */
   cursor = 0;
+  geom_i = -1;
   do {
     /* get the geometry */
     obj = *(GeometryObject**)dataptr[0];
+    geom_i++;
     if (!get_geom(obj, &geom)) {
       errstate = PGERR_NOT_A_GEOMETRY;
       goto finish;
@@ -421,11 +442,20 @@ PyObject* GetCoords(PyArrayObject* arr, int include_z) {
     if (geom == NULL) {
       continue;
     }
-    /* get the coordinates */
+    /* keep track of the current cursor in "i" to be able to loop from the
+    first to the last coordinate belonging to this geometry later */
+    i = cursor;
+    /* get the coordinates (updates "cursor") */
     if (!get_coordinates(ctx, geom, result, &cursor, include_z)) {
       errstate = PGERR_GEOS_EXCEPTION;
       goto finish;
     }
+    if (return_index) {
+      /* loop from "i" to "cursor" */
+      for (; i < cursor; i++) {
+        *(npy_intp*)PyArray_GETPTR1(index, i) = geom_i;
+      }
+    }
   } while (iternext(iter));
 
 finish:
@@ -434,7 +464,13 @@ finish:
 
   if (errstate != PGERR_SUCCESS) {
     Py_DECREF(result);
+    Py_XDECREF(index);
     return NULL;
+  } else if (return_index) {
+    PyObject* result_tpl = PyTuple_New(2);
+    PyTuple_SET_ITEM(result_tpl, 0, (PyObject*)result);
+    PyTuple_SET_ITEM(result_tpl, 1, (PyObject*)index);
+    return result_tpl;
   } else {
     return (PyObject*)result;
   }
@@ -464,7 +500,7 @@ PyObject* SetCoords(PyArrayObject* geoms, PyArrayObject* coords) {
   /* We use the Numpy iterator C-API here.
   The iterator exposes an "iternext" function which updates a "dataptr"
   see also: https://docs.scipy.org/doc/numpy/reference/c-api.iterator.html */
-  iter = NpyIter_New(geoms, NPY_ITER_READWRITE | NPY_ITER_REFS_OK, NPY_KEEPORDER,
+  iter = NpyIter_New(geoms, NPY_ITER_READWRITE | NPY_ITER_REFS_OK, NPY_CORDER,
                      NPY_NO_CASTING, NULL);
   if (iter == NULL) {
     return NULL;
@@ -541,8 +577,9 @@ PyObject* PyCountCoords(PyObject* self, PyObject* args) {
 PyObject* PyGetCoords(PyObject* self, PyObject* args) {
   PyObject* arr;
   int include_z;
+  int return_index;
 
-  if (!PyArg_ParseTuple(args, "Op", &arr, &include_z)) {
+  if (!PyArg_ParseTuple(args, "Opp", &arr, &include_z, &return_index)) {
     return NULL;
   }
   if (!PyArray_Check(arr)) {
@@ -553,7 +590,7 @@ PyObject* PyGetCoords(PyObject* self, PyObject* args) {
     PyErr_SetString(PyExc_TypeError, "Array should be of object dtype");
     return NULL;
   }
-  return GetCoords((PyArrayObject*)arr, include_z);
+  return GetCoords((PyArrayObject*)arr, include_z, return_index);
 }
 
 PyObject* PySetCoords(PyObject* self, PyObject* args) {
@@ -563,7 +600,7 @@ PyObject* PySetCoords(PyObject* self, PyObject* args) {
   if (!PyArg_ParseTuple(args, "OO", &geoms, &coords)) {
     return NULL;
   }
-  if ((!PyArray_Check(geoms)) | (!PyArray_Check(coords))) {
+  if ((!PyArray_Check(geoms)) || (!PyArray_Check(coords))) {
     PyErr_SetString(PyExc_TypeError, "Not an ndarray");
     return NULL;
   }
