@@ -4,6 +4,7 @@
 #include "geos.h"
 
 #include <Python.h>
+#include <numpy/npy_math.h>
 #include <structmember.h>
 
 /* This initializes a globally accessible GEOSException object */
@@ -15,18 +16,12 @@ int init_geos(PyObject* m) {
   return 0;
 }
 
-/* Returns 1 if geometry is an empty point, 0 otherwise, 2 on error.
- */
-char is_point_empty(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
-  int geom_type;
-
-  geom_type = GEOSGeomTypeId_r(ctx, geom);
-  if (geom_type == GEOS_POINT) {
-    return GEOSisEmpty_r(ctx, geom);
-  } else if (geom_type == -1) {
-    return 2;  // GEOS exception
-  } else {
-    return 0;  // No empty point
+void destroy_geom_arr(void* context, GEOSGeometry** array, int length) {
+  int i;
+  for (i = 0; i < length; i++) {
+    if (array[i] != NULL) {
+      GEOSGeom_destroy_r(context, array[i]);
+    }
   }
 }
 
@@ -55,6 +50,25 @@ char multipoint_has_point_empty(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
   return 0;
 }
 
+// POINT EMPTY is converted to POINT (nan nan)
+// by GEOS >= 3.10.0. Before that, we do it ourselves here.
+#if !GEOS_SINCE_3_10_0
+
+/* Returns 1 if geometry is an empty point, 0 otherwise, 2 on error.
+ */
+char is_point_empty(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
+  int geom_type;
+
+  geom_type = GEOSGeomTypeId_r(ctx, geom);
+  if (geom_type == GEOS_POINT) {
+    return GEOSisEmpty_r(ctx, geom);
+  } else if (geom_type == -1) {
+    return 2;  // GEOS exception
+  } else {
+    return 0;  // No empty point
+  }
+}
+
 /* Returns 1 if a geometrycollection has an empty point, 0 otherwise, 2 on error.
 Checks recursively (geometrycollections may contain multipoints / geometrycollections)
 */
@@ -133,15 +147,6 @@ GEOSGeometry* point_empty_to_nan(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
   return result;
 }
 
-void destroy_geom_arr(void* context, GEOSGeometry** array, int length) {
-  int i;
-  for (i = 0; i < length; i++) {
-    if (array[i] != NULL) {
-      GEOSGeom_destroy_r(context, array[i]);
-    }
-  }
-}
-
 /* Creates a new multipoint, replacing empty points with POINT (nan, nan[, nan)]
 
    Returns NULL on error
@@ -250,6 +255,8 @@ GEOSGeometry* point_empty_to_nan_all_geoms(GEOSContextHandle_t ctx, GEOSGeometry
   return result;
 }
 
+#endif  // !GEOS_SINCE_3_10_0
+
 /* Checks whether the geometry is a multipoint with an empty point in it
  *
  * According to https://github.com/libgeos/geos/issues/305, this check is not
@@ -284,13 +291,13 @@ char check_to_wkt_compatible(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
 
 /* GEOSInterpolate_r and GEOSInterpolateNormalized_r segfault on empty
  * geometries and also on collections with the first geometry empty.
- * 
+ *
  * This function returns:
  * - PGERR_GEOMETRY_TYPE on non-linear geometries
  * - PGERR_EMPTY_GEOMETRY on empty linear geometries
  * - PGERR_EXCEPTIONS on GEOS exceptions
  * - PGERR_SUCCESS on a non-empty and linear geometry
- * 
+ *
  * Note that GEOS 3.8 fixed this situation for empty LINESTRING/LINEARRING,
  * but it still segfaults on other empty geometries.
  */
@@ -303,7 +310,7 @@ char geos_interpolate_checker(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
   type = GEOSGeomTypeId_r(ctx, geom);
   if (type == -1) {
     return PGERR_GEOS_EXCEPTION;
-  } else if ((type == GEOS_POINT) | (type == GEOS_POLYGON) | (type == GEOS_MULTIPOINT) |
+  } else if ((type == GEOS_POINT) || (type == GEOS_POLYGON) || (type == GEOS_MULTIPOINT) ||
              (type == GEOS_MULTIPOLYGON)) {
     return PGERR_GEOMETRY_TYPE;
   }
@@ -317,7 +324,7 @@ char geos_interpolate_checker(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
   }
 
   // For collections: also check the type and emptyness of the first geometry
-  if ((type == GEOS_MULTILINESTRING) | (type == GEOS_GEOMETRYCOLLECTION)) {
+  if ((type == GEOS_MULTILINESTRING) || (type == GEOS_GEOMETRYCOLLECTION)) {
     sub_geom = GEOSGetGeometryN_r(ctx, geom, 0);
     if (sub_geom == NULL) {
       return PGERR_GEOS_EXCEPTION;  // GEOSException
@@ -325,7 +332,7 @@ char geos_interpolate_checker(GEOSContextHandle_t ctx, GEOSGeometry* geom) {
     type = GEOSGeomTypeId_r(ctx, sub_geom);
     if (type == -1) {
       return PGERR_GEOS_EXCEPTION;
-    } else if ((type != GEOS_LINESTRING) & (type != GEOS_LINEARRING)) {
+    } else if ((type != GEOS_LINESTRING) && (type != GEOS_LINEARRING)) {
       return PGERR_GEOMETRY_TYPE;
     }
     is_empty = GEOSisEmpty_r(ctx, sub_geom);
@@ -346,3 +353,202 @@ void geos_error_handler(const char* message, void* userdata) {
 void geos_notice_handler(const char* message, void* userdata) {
   snprintf(userdata, 1024, "%s", message);
 }
+
+/* Extract bounds from geometry.
+ *
+ * Bounds coordinates will be set to NPY_NAN if geom is NULL, empty, or does not have an
+ * envelope.
+ *
+ * Parameters
+ * ----------
+ * ctx: GEOS context handle
+ * geom: pointer to GEOSGeometry; can be NULL
+ * xmin: pointer to xmin value
+ * ymin: pointer to ymin value
+ * xmax: pointer to xmax value
+ * ymax: pointer to ymax value
+ *
+ * Must be called from within a GEOS_INIT_THREADS / GEOS_FINISH_THREADS
+ * or GEOS_INIT / GEOS_FINISH block.
+ *
+ * Returns
+ * -------
+ * 1 on success; 0 on error
+ */
+int get_bounds(GEOSContextHandle_t ctx, GEOSGeometry* geom, double* xmin, double* ymin,
+               double* xmax, double* ymax) {
+  int retval = 1;
+
+  if (geom == NULL || GEOSisEmpty_r(ctx, geom)) {
+    *xmin = *ymin = *xmax = *ymax = NPY_NAN;
+    return 1;
+  }
+
+#if GEOS_SINCE_3_7_0
+  // use min / max coordinates
+
+  if (!(GEOSGeom_getXMin_r(ctx, geom, xmin) && GEOSGeom_getYMin_r(ctx, geom, ymin) &&
+        GEOSGeom_getXMax_r(ctx, geom, xmax) && GEOSGeom_getYMax_r(ctx, geom, ymax))) {
+    return 0;
+  }
+
+#else
+  // extract coordinates from envelope
+
+  GEOSGeometry* envelope = NULL;
+  const GEOSGeometry* ring = NULL;
+  const GEOSCoordSequence* coord_seq = NULL;
+  int size;
+
+  /* construct the envelope */
+  envelope = GEOSEnvelope_r(ctx, geom);
+  if (envelope == NULL) {
+    return 0;
+  }
+
+  size = GEOSGetNumCoordinates_r(ctx, envelope);
+
+  /* get the bbox depending on the number of coordinates in the envelope */
+  if (size == 0) { /* Envelope is empty */
+    *xmin = *ymin = *xmax = *ymax = NPY_NAN;
+  } else if (size == 1) { /* Envelope is a point */
+    if (!GEOSGeomGetX_r(ctx, envelope, xmin)) {
+      retval = 0;
+      goto finish;
+    }
+    if (!GEOSGeomGetY_r(ctx, envelope, ymin)) {
+      retval = 0;
+      goto finish;
+    }
+    *xmax = *xmin;
+    *ymax = *ymin;
+  } else if (size == 5) { /* Envelope is a box */
+    ring = GEOSGetExteriorRing_r(ctx, envelope);
+    if (ring == NULL) {
+      retval = 0;
+      goto finish;
+    }
+    coord_seq = GEOSGeom_getCoordSeq_r(ctx, ring);
+    if (coord_seq == NULL) {
+      retval = 0;
+      goto finish;
+    }
+    if (!GEOSCoordSeq_getX_r(ctx, coord_seq, 0, xmin)) {
+      retval = 0;
+      goto finish;
+    }
+    if (!GEOSCoordSeq_getY_r(ctx, coord_seq, 0, ymin)) {
+      retval = 0;
+      goto finish;
+    }
+    if (!GEOSCoordSeq_getX_r(ctx, coord_seq, 2, xmax)) {
+      retval = 0;
+      goto finish;
+    }
+    if (!GEOSCoordSeq_getY_r(ctx, coord_seq, 2, ymax)) {
+      retval = 0;
+      goto finish;
+    }
+  }
+
+finish:
+  if (envelope != NULL) {
+    GEOSGeom_destroy_r(ctx, envelope);
+  }
+
+#endif
+
+  return retval;
+}
+
+/* Create a Polygon from bounding coordinates.
+ *
+ * Must be called from within a GEOS_INIT_THREADS / GEOS_FINISH_THREADS
+ * or GEOS_INIT / GEOS_FINISH block.
+ *
+ * Parameters
+ * ----------
+ * ctx: GEOS context handle
+ * xmin: minimum X value
+ * ymin: minimum Y value
+ * xmax: maximum X value
+ * ymax: maximum Y value
+ * ccw: if 1, box will be created in counterclockwise direction from bottom right;
+ *  otherwise will be created in clockwise direction from bottom left.
+ *
+ * Returns
+ * -------
+ * GEOSGeometry* on success (owned by caller) or
+ * NULL on failure or NPY_NAN coordinates
+ */
+GEOSGeometry* create_box(GEOSContextHandle_t ctx, double xmin, double ymin, double xmax,
+                         double ymax, char ccw) {
+  if (npy_isnan(xmin) || npy_isnan(ymin) || npy_isnan(xmax) || npy_isnan(ymax)) {
+    return NULL;
+  }
+
+  GEOSCoordSequence* coords = NULL;
+  GEOSGeometry* geom = NULL;
+  GEOSGeometry* ring = NULL;
+
+  // Construct coordinate sequence and set vertices
+  coords = GEOSCoordSeq_create_r(ctx, 5, 2);
+  if (coords == NULL) {
+    return NULL;
+  }
+
+  if (ccw) {
+    // Start from bottom right (xmax, ymin) to match shapely
+    if (!(GEOSCoordSeq_setX_r(ctx, coords, 0, xmax) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 0, ymin) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 1, xmax) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 1, ymax) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 2, xmin) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 2, ymax) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 3, xmin) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 3, ymin) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 4, xmax) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 4, ymin))) {
+      if (coords != NULL) {
+        GEOSCoordSeq_destroy_r(ctx, coords);
+      }
+
+      return NULL;
+    }
+  } else {
+    // Start from bottom left (min, ymin) to match shapely
+    if (!(GEOSCoordSeq_setX_r(ctx, coords, 0, xmin) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 0, ymin) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 1, xmin) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 1, ymax) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 2, xmax) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 2, ymax) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 3, xmax) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 3, ymin) &&
+          GEOSCoordSeq_setX_r(ctx, coords, 4, xmin) &&
+          GEOSCoordSeq_setY_r(ctx, coords, 4, ymin))) {
+      if (coords != NULL) {
+        GEOSCoordSeq_destroy_r(ctx, coords);
+      }
+
+      return NULL;
+    }
+  }
+
+  // Construct linear ring then use to construct polygon
+  // Note: coords are owned by ring; if ring fails to construct, it will
+  // automatically clean up the coords
+  ring = GEOSGeom_createLinearRing_r(ctx, coords);
+  if (ring == NULL) {
+    return NULL;
+  }
+
+  // Note: ring is owned by polygon; if polygon fails to construct, it will
+  // automatically clean up the ring
+  geom = GEOSGeom_createPolygon_r(ctx, ring, NULL, 0);
+  if (geom == NULL) {
+    return NULL;
+  }
+
+  return geom;
+}
