@@ -126,7 +126,8 @@ static PyObject* GeometryObject_ToWKB(GeometryObject* obj) {
 
   GEOS_INIT;
 
-  // WKB Does not allow empty points.
+#if !GEOS_SINCE_3_10_0
+  // WKB Does not allow empty points in GEOS < 3.10.
   // We check for that and patch the POINT EMPTY if necessary
   has_empty = has_point_empty(ctx, obj->ptr);
   if (has_empty == 2) {
@@ -138,6 +139,9 @@ static PyObject* GeometryObject_ToWKB(GeometryObject* obj) {
   } else {
     geom = obj->ptr;
   }
+#else
+  geom = obj->ptr;
+#endif  // !GEOS_SINCE_3_10_0
 
   /* Create the WKB writer */
   writer = GEOSWKBWriter_create_r(ctx);
@@ -164,7 +168,7 @@ static PyObject* GeometryObject_ToWKB(GeometryObject* obj) {
 
 finish:
   // Destroy the geom if it was patched (POINT EMPTY patch)
-  if (has_empty & (geom != NULL)) {
+  if (has_empty && (geom != NULL)) {
     GEOSGeom_destroy_r(ctx, geom);
   }
   if (writer != NULL) {
@@ -421,7 +425,7 @@ int __Pyx_InBases(PyTypeObject* a, PyTypeObject* b) {
 Py_None. Returns 0 on error, 1 on success. */
 char get_geom(GeometryObject* obj, GEOSGeometry** out) {
   PyTypeObject* type = ((PyObject*)obj)->ob_type;
-  if ((type != &GeometryType) & !(__Pyx_InBases(type, &GeometryType))) {
+  if ((type != &GeometryType) && !(__Pyx_InBases(type, &GeometryType))) {
     if ((PyObject*)obj == Py_None) {
       *out = NULL;
       return 1;
